/////////////////////////////////////////////////////////////////////////////
// File:            libGenome/gnVersion.h
// Purpose:         Defines libGenome version info
// Description:     Defines Version info
// Rev:             A
// Author:          Aaron Darling 
// Modified by:     
// Copyright:       (c) Aaron Darling 
// Licenses:        See COPYING file for details 
/////////////////////////////////////////////////////////////////////////////
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef _gnVersion_h_
#define _gnVersion_h_

/* Increment with each new version */
/* Build number with each build, minor with minor changes, major with major API changes */
#define gnMAJOR_VERSION		0
#define gnMINOR_VERSION		5
#define gnBUILD_NUMBER		1
#define gnVERSION_STRING	"libGenome 0.5.1"
#define gnVERSION_NUMBER	(gnMAJOR_VERSION * 1000) + (gnMINOR_VERSION * 100) + gnRELEASE_NUMBER
#define gnBETA_NUMBER		1
#define gnVERSION_FLOAT		gnMAJOR_VERSION + (gnMINOR_VERSION/10.0) + (gnRELEASE_NUMBER/100.0) + (gnBETA_NUMBER/10000.0)

#endif
	//_gnVersion_h_
